###########
# obsolete
compoisson.z <- function(lambda,nu) {
  .C("compoisson_z",lambda=as.double(lambda),nu=as.double(nu))$lambda
}
compoisson.mean <- function(lambda,nu) {
  .C("compoisson_mean",lambda=as.double(lambda),nu=as.double(nu))$lambda
}
compoisson.var <- function(lambda,nu) {
  m = compoisson.mean(lambda,nu)
  m2 = .C("compoisson_m2",lambda=as.double(lambda),nu=as.double(nu))$lambda
  m2 - m^2
}

#############
compoisson.z.ln <- function(lambda,nu) {
  x = c()
  for(i in seq(length(lambda))) {
    r = .C("compoisson_z_ln",lambda=as.double(lambda[i]),nu=as.double(nu))
    if(r$nu == 10000) stop("not enough iters")
    #cat("compoisson.z.ln:",x$nu,"iters\n")
    x[i] = r$lambda
  }
  x
}
compoisson.zx.ln <- function(lambda,nu,p) {
  x = c()
  for(i in seq(length(lambda))) {
    r = .C("compoisson_zx_ln",lambda=as.double(lambda[i]),nu=as.double(nu),
      p=as.double(p))
    if(r$nu == 10000) stop("not enough iters")
    #cat("compoisson.zx.ln:",x$nu,"iters\n")
    x[i] = r$lambda
  }
  x
}
compoisson.z.ln.approx <- function(lambda,nu) {
  if(nu == 0) {
    # should be -log(1-lambda)
    if(lambda > 1) Inf
    else -Inf
  } else {
    z = nu*lambda^(1/nu) - (nu-1)/2/nu*log(lambda) -
      (nu-1)/2*log(2*pi) - 0.5*log(nu)
    if(F) z = z + log(1 + (nu-1)*(nu+1)/24/nu/lambda^(1/nu))
    z
  }
}
compoisson.mean <- function(lambda,nu) {
  exp(compoisson.zx.ln(lambda,nu,1) - compoisson.z.ln(lambda,nu))
}
compoisson.mean.approx <- function(lambda,nu) {
  lambda^(1/nu) - (nu-1)/2/nu
}
compoisson.var <- function(lambda,nu) {
  m2 = exp(compoisson.zx.ln(lambda,nu,2) - compoisson.z.ln(lambda,nu))
  m2 - compoisson.mean(lambda,nu)^2
}
compoisson.elfact.approx <- function(lambda,nu) {
  1/2/nu^2*log(lambda) + lambda^(1/nu)*(log(lambda)/nu - 1)
}

compoisson.mean.fit <- function(m,nu) {
  # returns lambda such that compoisson.mean(lambda,nu) == m
  if(nu == 0) {
    return(m/(m+1))
  } else if(nu == 1) {
    return(m)
  }
  # initialize
  m.nu = m + (nu-1)/2/nu
  if(m.nu > 1) lambda = m.nu^nu
  else {
    a = 2/2^nu - 1
    lambda = (sqrt(1 + 4*a*m)-1)/2/a
    # is lambda == 1 better?
    if(abs(compoisson.mean(1-1e-5,nu) - m) < abs(compoisson.mean(lambda,nu) - m)) {
      lambda = 1-1e-5
    }
  }
  # Newton iterations
  for(iter in 1:100) {
    #cat("iter",iter,": ",lambda,"\n")
    if(abs(lambda) < 1e-16) { lambda = 0; break }
    if(lambda < 0) stop("compoisson.mean.fit failed")
    # derivative of mean wrt lambda is var/lambda
    delta = lambda*(m - compoisson.mean(lambda,nu))/compoisson.var(lambda,nu)
    lambda = lambda + delta
    if(abs(delta) < 1e-10) break
  }
  lambda
}

compoisson.fit <- function(x,positive=F) {
  # returns ML parameters given data x.
  # x is a vector of counts, starting from 0.
  # if positive=T, fits a positive COM-Poisson (cf Wimmer et al)
  # if positive=T, the first count in x must be zero.
  i = (1:length(x))-1
  dx = x/sum(x)
  #dx[dx==0] = 1e-16
  compoisson.fcn <- function(theta) {
    lambda = theta[1]
    nu = theta[2]
    if(lambda < 0 || nu < 0) return(1e10)
    d = dcompoisson(i,lambda,nu)+1e-16
    if(positive) {
      d[1] = 0
      d = d/sum(d)
    }
    sum(x*log(dx/d),na.rm=T)
  }
  nu = 1
  lambda = sum(i*x)/sum(x)
  theta = c(lambda,nu)
  theta = nlm(compoisson.fcn, theta)$estimate
  list(lambda=theta[1],nu=theta[2])
}

dcompoisson <- function(x,lambda,nu,log=F) {
  # compute COM-Poisson probabilities
  d = x*log(lambda) - nu*lgamma(x+1) - compoisson.z.ln(lambda,nu)
  if(!log) d = exp(d)
  d
}

jsd <- function(lp1,lp2) {
  .C("jsd",as.integer(length(lp1)),
     as.double(lp1),as.double(lp2),result=double(1))[["result"]]
}

compoisson.plot <- function(p,ylim=NULL,asp=1,main="COM-Poissonness plot",...) {
  x = 0:(length(p)-1)
  m = sum(x*p)/sum(p)
  if(F) {
    # pilot geometric estimate
    a = dgeom(x,1/(m+1))
    p = p + a
  }
  i = 1:(length(p)-1)
  p[p == 0] = NA
  y = log(p[i]/p[i+1])
  z = log(x[i+1])
  v = 1/p[i] + 1/p[i+1]
  n = length(y)
  vc = array(0,c(n,n))
  for(j in 1:(n-1)) {
    vc[j,j+1] = -1/p[j+1]
    vc[j+1,j] = vc[j,j+1]
  }
  diag(vc) = v

  keep = !is.na(y)
  y = y[keep]
  z = z[keep]
  v = v[keep]
  vc = vc[keep,keep]

  # WLS
  # y ~ N(a*z1, vc)
  z1 = rbind(rep(1,length(z)),z)
  vc = solve(vc)
  a = drop(solve(z1 %*% vc %*% t(z1), z1 %*% vc %*% y))
  names(a) = c("intercept","slope")
  
  se = 1.64*sqrt(v)
  if(is.null(ylim)) {
    ylim = c(min(y - se,na.rm=T), max(y + se,na.rm=T))
  } else if(ylim == "tight") {
    ylim = range(y)
  }
  plot(z,y,xlab="ln(x)",ylab="ln( #[x-1] / #x )",ylim=ylim,asp=asp,...)
  title(main=main)
  arrows(z,y-se,z,y+se,len=0.01,angle=90,code=3)
  abline(-log(m),1,lty="dotted")
  abline(a[1],a[2])

  a[1] = exp(-a[1])
  names(a) = c("lambda","nu")
  labels = c("Data","Poisson","COM")
  labels[2] = paste(labels[2],"(",format(m,digits=3),")",sep="")
  labels[3] = paste(labels[3],"(",format(a[1],digits=3),",",format(a[2],digits=3),")",sep="")
  legend(par("usr")[1],par("usr")[4],labels,lty=c(NA,3,1),pch=c(1,NA,NA))
  a
}
