# see tex/nu_poisson/notes

consul.fit <- function(x,w) {
  # only works for a>0
  if(missing(w)) w = rep(1,length(x))
  n = sum(w)
  # initialize with moment match
  m = sum(w*x)/n
  v = sum(w*(x-m)^2)/n
  a = 1 - sqrt(m/v)
  b = m*(1-a)
  for(iter in 1:100) {
    old.a = a
    q = 1 - b/(b+a*x)
    a = sum(w*(x-1)*q)/(n*m)
    b = m*(1-a)
    if(abs(a - old.a) < 1e-6) break
  }
  if(iter == 100) warning("not enough iters")
  list(a=a,b=b)
}
consul.fit <- function(x,w) {
  # works for all a
  if(missing(w)) w = rep(1,length(x))
  i.nz = (w > 0)
  x = x[i.nz]
  w = w[i.nz]
  n = sum(w)
  # initialize with moment match
  m = sum(w*x)/n
  v = sum(w*(x-m)^2)/n
  a = 1 - sqrt(m/v)
  i1 = (x>=m)
  i0 = (x<m)
  a.min = m/(m - max(x))
  for(iter in 1:100) {
    old.a = a
    d = m + a*(x-m)
    q1 = (a-a.min)*(x[i1]-m)/d[i1]
    q0 = (1-a)*(m-x[i0])/d[i0]
    c1 = sum(w[i1]*(x[i1]-1)*q1)
    c0 = n + sum(w[i0]*(x[i0]-1)*q0)
    a = (c1 + a.min*c0)/(c1 + c0)
    if(abs(a - old.a) < 1e-6) break
  }
  if(iter == 100) warning("not enough iters")
  b = m*(1-a)
  list(a=a,b=b)
}

dconsul <- function(x,a,b,log=F) {
  if(a < 0 || a >= 1) stop("must have 0 <= a < 1")
  if(b < 0) stop("must have b > 0")
  d = log(b) + (x-1)*log(b+a*x) -b-a*x - lgamma(x+1)
  d[b+a*x < 0] = -Inf
  if(!log) d = exp(d)
  d
}

consul.mean.fit <- function(m,a) {
  m*(1-a)
}

consul.mean <- function(a,b) {
  b/(1-a)
}
consul.var <- function(a,b) {
  b/(1-a)^3
}
consul.dispersion.fit <- function(dis) {
  # dispersion is v/m = 1/(1-a)^2
  1 - 1/sqrt(dis)
}

dlingappaiah <- function(x,a,b,c,log=F) {
  if(a < 0 || a >= 1) stop("must have 0 <= a < 1")
  if(b < 0) stop("must have b > 0")
  if(c < 0 || c >= 1-a) stop("must have 0 <= c < 1-a")
  d = log(1-a-c)-log(1-a)+log(b+c*x) +
    (x-1)*log(b+(a+c)*x) -(b+(a+c)*x) - lgamma(x+1)
  #d[b+(a+c)*x < 0] = -Inf
  if(!log) d = exp(d)
  d
}

lingappaiah.mean <- function(a,b,c) {
  b/(1-a)
}

if(F) {
  # test
  a = 0.5
  b = 0.3
  c = 0.1
  x = 0:100
  p = dlingappaiah(x,a,b,c)
  plot(x,p,type="l")
  print(sum(p))
  m = sum(x*p)
  m.true = lingappaiah.mean(a,b,c)
  cat("true mean =",m.true,"est mean =",m,"\n")
}
