# Jain-Consul generalized negative binomial
# r=0 is ordinary NB
# p = b/(1+b)
# usually must have r>=1, p*r < 1
# see Johnson, Kotz, and Kemp 1992: p230

dgnb <- function(x,n,b,r,log=F) {
  if(r*b/(1+b) >= 1) stop("invalid parameter settings")
  lp = log(n)-log(n + r*x)+lgamma(n+r*x+1)-lgamma(x+1)-lgamma(n+r*x-x+1) +
    x*log(b) - (n+r*x)*log(1+b)
  if(log) lp else exp(lp)
}

gnb.mean <- function(n,b,r) {
  n*b/(1+b-r*b)
}

gnb.mean.fit <- function(m,b,r) {
  # returns value for n
  m*(1+b-r*b)/b
}

gnb.var <- function(n,b,r) {
  gnb.mean(n,b,r)*(1+b)/(1+b-r*b)^2
}

gnb.dispersion.fit <- function(dis,r) {
  # returns value for b
  # dis is desired value of var/mean
  if(r == 0) (1/dis - 1)
  else if(r == 1) (dis - 1)
  else ((2*(r-1)+1/dis) - sqrt((2*(r-1)+1/dis)^2 + 4*(r-1)^2*(1/dis-1)))/2/(r-1)^2
}

if(F) {
  # test
  library(nupois)
  x <- (1:5000)-1
  r = 2
  b = gnb.dispersion.fit(60/12,r)
  #b = gnb.dispersion.fit(0.9,r)
  n = gnb.mean.fit(12,b,r)
  p = dgnb(x,n,b,r)
  plot(p[1:50],type="l")
  cat("sum(p)=",sum(p),"\n")
  p = p/sum(p)
  m = sum(p*x)
  m.true = gnb.mean(n,b,r)
  cat("true mean =",m.true,"est mean =",m,"\n")
  v <- sum(p*x*x) - m^2
  cat("true var =",gnb.var(n,b,r),"est var =",v,"\n")
}
