gppf1 <- function(r, beta) {
  part1 <- (1 + beta)^(1 + r)
  part2 <- part1 - (1 + beta)
  return((r * beta)/part2)
}


gppfk <- function(k, r, beta, prevf) {
  return(((k + r - 1)/k * beta)/(1 + beta) * prevf)
}

gppf <- function(length,lambda,r,beta){
  answer = rep(0,length)
  answer[1] = gppf1(r,beta)
  for(i in c(2:length)) {
    answer[i] = gppfk(i,r,beta,answer[i-1])
  }
  return(answer)
}

gpp.slow <- function(length,lambda,r,beta) {
  # returns gpp density evaluated at 0..length-1
  fvec <- gppf(length-1,lambda,r,beta)
  answer = rep(0,length)
  answer[1] = exp(-lambda)
  for(i in c(2:length)) {
    cumsum = 0.0
    for(k in c(2:i)) {
      cumsum = cumsum+(k-1)*fvec[k-1]*answer[i-k+1]
    }
    answer[i] = lambda/(i-1)*cumsum
  }
  return(answer)
}

gpp <- function(length,lambda,r,beta,log=F) {
  if(lambda <= 0) stop("must have lambda > 0")
  if(r <= -1) stop("must have r > -1")
  if(beta <= 0) stop("must have beta > 0")
  ans = .C("gpp",as.integer(length),as.double(lambda),as.double(r),
                 as.double(beta),ans=double(length))$ans
  if(!log) ans = exp(ans)
  ans
}

dgpp <- function(x,lambda,r,beta,log=F) {
  p = gpp(max(x)+1,lambda,r,beta,log)
  p[x+1]
}

gpp.mean <- function(lambda,r,beta) {
  if(r == 0) lambda*beta/log(1+beta)
  else lambda*r*beta/(1-(1+beta)^(-r))
}

gpp.mean.fit <- function(m,r,beta) {
  if(r == 0) lambda = m/(beta/log(1+beta))
  else lambda = m/(r*beta/(1-(1+beta)^(-r)))
  lambda
}

gpp.fit <- function(x) {
  dx = x/sum(x)
  dx[dx==0] = 1e-16
  cost.fcn <- function(theta) {
    #print(r)
    lambda = theta[1]
    beta = theta[3]
    r = theta[2]
    if(lambda <= 0 || beta <= 0 || r <= -1) return(1e10)
    d = gpp(length(x),lambda,r,beta)
    sum(x*log(dx/d))
  }
  # initialize with r=0
  i = seq(0,length(x)-1)
  m = sum(i*x)/sum(x)
  v = sum((i-m)^2*x)/sum(x)
  # index of dispersion
  dis = v/m
  if(dis > 1) beta = dis-1
  else beta = 0.1
  theta = c(m/(beta/log(1+beta)),0,beta)
  theta = nlm(cost.fcn, theta)$estimate
  list(lambda=theta[1],r=theta[2],beta=theta[3])
}

if(F) {
  library(nupois)
  x <- (1:100)-1
  lambda <- 1
  r <- -1
  beta <- 0.1
  p <- dgpp(x, lambda, r, beta)
  sum(p)
  p <- p/sum(p)
  m <- sum(p*x)
  m.true = gpp.mean(lambda,r,beta)
  cat("true mean =",m.true,"est mean =",m,"\n")
  v <- sum(p*x*x) - m^2
  cat("true var =",gpp.var(lambda,r,beta),"est var =",v,"\n")
}

gpp.var <- function(lambda,r,beta) {
  gpp.mean(lambda,r,beta)*(1+(r+1)*beta)
}

gpp.dispersion.fit <- function(dis,r) {
  # returns value for beta
  # dis is desired value of var/mean
  (dis-1)/(r+1)
}
