gamma.fit <- function(x,s) {
  # x is vector of data.  s is optional.
  if(missing(s)) {
    m <- mean(x)
    s <- log(m) - mean(log(x))
  } else {
    m <- x
  }
  a <- 0.5/s
  for(iter in 1:100) {
    old.a <- a
    g <- log(a) - s - digamma(a)
    h <- 1/a - trigamma(a)
    a <- 1/(1/a + g/(a^2*h))
    if(abs(a - old.a) < 1e-8) break
  }
  b <- m/a
  list(a=a,b=b)
}

negbin.fit <- function(x,t) {
  fit <- glm.nb(x ~ offset(log(t)))
  a0 <- fit$theta
  b0 <- exp(coef(fit))/a0
  list(a=a0,b=b0)
}
# faster version
negbin.fit <- function(x,t,w) {
  # x is vector of data.  t is optional.
  # w[i] is the weight of x[i].
  if(missing(t)) t <- rep(1,length(x))
  # initialize
  xt <- x/t
  if(missing(w)) {
    v <- var(xt)
    m <- mean(xt)
  } else {
    w <- w/sum(w)
    m <- sum(w*xt)
    v <- sum(w*(xt - m)^2)
  }
  b <- v/m - 1
  if(b < 0) b <- 1e-3
  a <- m/b
  # EM
  for(iter in 1:100) {
    old.a <- a
    # E step
    p <- b/(1+b*t)
    if(missing(w)) {
      m <- mean((x+a)*p)
      s <- log(m) - mean(digamma(x+a) + log(p))
    } else {
      m <- sum(w*(x+a)*p)
      s <- log(m) - sum(w*(digamma(x+a) + log(p)))
    }
    # M step
    r <- gamma.fit(m,s)
    a <- r$a
    b <- r$b
    if(abs(a - old.a) < 1e-8) break
  }
  r
}

