#include <malloc.h>
#include <math.h>

double logsumexp(double a, double b);

double gppf1(double r, double beta)
{
  double b = beta/(1+beta);
  if(r == 0) {
    return(b/log(1+beta));
  } 
  else {
    return(b*r/(pow(1+beta, r) - 1));
  }
}

double gppfk(double k, double r, double beta)
{
  return(((k + r - 1)/k * beta)/(1 + beta));
}

double *gppf(int length, double lambda, double r, double beta)
{
  int i;
  double *ans = (double*)malloc(length*sizeof(double));
  ans[0] = log(gppf1(r,beta));
  for(i=1;i<length;i++) {
    ans[i] = ans[i-1] + log(gppfk(i+1,r,beta));
  }
  return(ans);
}

void gpp(int *length, double *lambda, double *r, double *beta, double *ans)
{
  int i,j;
  double *fvec = gppf(*length-1,*lambda,*r,*beta);
  ans[0] = -*lambda;
  for(i=1;i<*length;i++) {
    double sum = fvec[0]+ans[i-1];
    for(j=2;j<=i;j++) {
      sum = logsumexp(sum,log(j)+fvec[j-1]+ans[i-j]);
    }
    ans[i] = log(*lambda)-log(i)+sum;
  }
  free(fvec);
}

