# Fits various distributions to a data set.

library(compoisson)
source("compoisson/R/compoisson.r")
source("compoisson/R/morlat.r")

# fardally data:
x<-c(1,1,0,2,2,4,7,7,8,0,0,2,3,4,3)
if(F) {
  data(gdata)
  #x = gdata[["Strike"]]
  #x = gdata[["Clutch"]]
  x = gdata[[12]]
}
positive = F
if(positive) x = c(0,x)
#positive = F

i <- (1:length(x))-1
frame = list()

frame[["Data"]] = x

good.ones = T
ylim = NULL
#ylim = c(-60,60)
save = F

if(T) {
  # Poisson
  m = sum(i*x)/sum(x)
  d = dpois(i,m)
  cat("Poisson log-likelihood = ",sum(x*log(d)),"\n")
  frame[["Poisson"]] = d*sum(x)

  # Geometric
  d = dgeom(i,1/(m+1))
  cat("Geometric log-likelihood = ",sum(x*log(d)),"\n")
  frame[["Geometric"]] = d*sum(x)
}

if(T) {
  # Consul's Negative Binomial
  r = cnb.fit(x)
  #print(r)
  d = dcnb(i, r$a,r$b,r$k)
  cat("CNB log-likelihood = ",sum(x*log(d)),"\n")
  frame[["CNB"]] = d*sum(x)
}

if(T) {
  if(T) {
    # Negative Binomial
    r = negbin.fit(i,w=x)
    d = dnbinom(i, r$a, mu=r$a*r$b)
    cat("NB log-likelihood = ",sum(x*log(d)),"\n")
    frame[["Negative Binomial"]] = d*sum(x)
  }
  # Consul-Jain Poisson
  r = consul.fit(i,w=x)
  if(r$a >= 0) {
    d = dconsul(i, r$a, r$b)
    cat("Consul log-likelihood = ",sum(x*log(d)),"\n")
    frame[["Generalized Poisson"]] = d*sum(x)
  }

  if(T) {
    # Willmot
    r = gpp.fit(x)
    #print(r)
    d = dgpp(i, r$lambda, r$r, r$beta)
    cat("GPP log-likelihood = ",sum(x*log(d)),"\n")
    frame[["GPP"]] = d*sum(x)
  }

  # Conway-Maxwell Poisson
  r = compoisson.fit(x,positive=positive)
  print(r)
  d = dcompoisson(i, r$lambda, r$nu)
  like = sum(x*log(d))
  if(positive) {
    d[1] = 0
    d = d/sum(d)
    like = sum(x[-1]*log(d[-1]))
  }
  cat("COM log-likelihood = ",like,"\n")
  frame[["COM"]] = d*sum(x)
}

#frame = nocheck.data.frame(frame,check=F)
frame = data.frame(frame)
frame = t(as.matrix(frame))
colnames(frame) = i

if(("mining" %in% .packages(all=T)) && require(mining,quiet=T)) {
  par(mfrow=c(1,1))
  par(mar=auto.mar(main="foo",xlab="",ylab=""))
  linechart(frame,legend=c(1,1))
  if(F) {
    frame = frame[-1,] - rep.row(frame[1,],nrow(frame)-1)
    linechart(frame,legend=2,ylab="Fitted Count - Observed Count")
  }
}
if(F) {
  # figures for paper
  #par(mfrow=auto.layout(nrow(frame)-1))
  for(i in 2:nrow(frame)) {
    subframe = frame[c(1,i),]
    shortname = abbreviate(rownames(subframe)[2],3)

    fname = paste("bars_",shortname,".eps",sep="")
    if(save) postscript(fname,width=8,height=6,horiz=F,paper="special")
    #linechart(subframe,legend=2,col=c(2,1))
    barplot(subframe,beside=T,col=c(1,0),legend.text=rownames(subframe),axis.lty=1)
    #linechart(frame[c(1,i),],legend=2,color.pal=1,lty=c(7,1))
    if(save) dev.off()
    #dev.copy2eps(file=fname)

    if(T) {
      fname = paste("resid_",shortname,".eps",sep="")
      if(save) postscript(fname,width=8,height=6,horiz=F,paper="special")
      ylab = paste(rownames(subframe)[2],"residuals")
      if(is.null(ylim))
        mp=barplot(subframe[2,]-subframe[1,],main=ylab,col=1,axis.lty=1)
      else
        mp=barplot(subframe[2,]-subframe[1,],main=ylab,ylim=ylim,col=1,axis.lty=1)
      #abline(v=mp,lty="dotted",col="lightgray")
      if(save) dev.off()
    }
  }
  # dev.copy2eps(file="clothing-diff2.eps")
  # dev.copy2eps(file="clothing-bars.eps")
  # dev.copy2eps(file="clothing-lines.eps")
}
